import numpy as np
from py_ti import catch_errors
from py_ti.function_helper import fill_for_noncomputable_vals
from py_ti.simple_moving_average import (
    simple_moving_average as sma
    )


def percent_k(data, period):
    """
    %K
    """
    catch_errors.check_for_period_error(data, period)
    percent_k = map(
        lambda idx:
        ((data[idx] - np.min(data[idx+1-period:idx+1])) /
         (np.max(data[idx+1-period:idx+1]) -
          np.min(data[idx+1-period:idx+1]))),
        range(period-1, len(data))
        )
    percent_k = fill_for_noncomputable_vals(data, percent_k)

    return percent_k


def percent_d(data, period):
    """
    %D
    """
    p_k = percent_k(data, period)
    percent_d = sma(p_k, 3)
    return percent_d



from py_ti import catch_errors
from py_ti.exponential_moving_average import (
    exponential_moving_average as ema
    )


def triple_exponential_moving_average(data, period):
    """
    Triple Exponential Moving Average

    TEMA = (3*EMA - 3*EMA(EMA)) + EMA(EMA(EMA))
    """
    catch_errors.check_for_period_error(data, period)

    tema = ((3 * ema(data, period) - (3 * ema(ema(data, period), period))) +
            ema(ema(ema(data, period), period), period)
            )
    return tema



import numpy as np
from py_ti import catch_errors


def typical_price(close_data, high_data, low_data):
    """
    Typical Price.
    TPt = (HIGHt + LOWt + CLOSEt) / 3
    """
    catch_errors.check_for_input_len_diff(close_data, high_data, low_data)
    tp = map(
        lambda idx:
        (high_data[idx] + low_data[idx] + close_data[idx]) / 3,
        range(0, len(close_data))
        )
    return np.array(tp)



from py_ti import catch_errors
from py_ti.exponential_moving_average import (
    exponential_moving_average as ema
    )


def double_exponential_moving_average(data, period):
    """
    Double Exponential Moving Average.

    DEMA = 2*EMA - EMA(EMA)
    """
    catch_errors.check_for_period_error(data, period)

    dema = (2 * ema(data, period)) - ema(ema(data, period), period)
    return dema



import numpy as np
from py_ti import catch_errors
from py_ti.smoothed_moving_average import (
    smoothed_moving_average as smma
    )
from py_ti.average_true_range import (
    average_true_range as atr
    )


def calculate_up_moves(high_data):
    """
    Up Move.
    UPMOVE = Ht - Ht-1
    """
    up_moves = map(
        lambda idx: high_data[idx] - high_data[idx-1],
        range(1, len(high_data))
        )
    return [np.nan] + up_moves


def calculate_down_moves(low_data):
    """
    Down Move.
    DWNMOVE = Lt-1 - Lt
    """
    down_moves = map(
        lambda idx: low_data[idx-1] - low_data[idx],
        range(1, len(low_data))
        )
    return [np.nan] + down_moves


def positive_directional_movement(high_data, low_data):
    """
    Positive Directional Movement (+DM)
    +DM: if UPMOVE > DWNMOVE and UPMOVE > 0 then +DM = UPMOVE else +DM = 0
    """
    catch_errors.check_for_input_len_diff(high_data, low_data)
    up_moves = calculate_up_moves(high_data)
    down_moves = calculate_down_moves(low_data)

    pdm = []
    for idx in range(0, len(up_moves)):
        if up_moves[idx] > down_moves[idx] and up_moves[idx] > 0:
            pdm.append(up_moves[idx])
        else:
            pdm.append(0)

    return pdm


def negative_directional_movement(high_data, low_data):
    """
    Negative Directional Movement (-DM)
    -DM: if DWNMOVE > UPMOVE and DWNMOVE > 0 then -DM = DWNMOVE else -Dm = 0
    """
    catch_errors.check_for_input_len_diff(high_data, low_data)
    up_moves = calculate_up_moves(high_data)
    down_moves = calculate_down_moves(low_data)

    ndm = []
    for idx in range(0, len(down_moves)):
        if down_moves[idx] > up_moves[idx] and down_moves[idx] > 0:
            ndm.append(down_moves[idx])
        else:
            ndm.append(0)

    return ndm


def positive_directional_index(close_data, high_data, low_data, period):
    """
    Positive Directional Index (+DI)
    +DI = 100 * SMMA(+DM) / ATR
    """
    catch_errors.check_for_input_len_diff(close_data, high_data, low_data)
    pdi = (100 *
           smma(positive_directional_movement(high_data, low_data), period) /
           atr(close_data, period)
           )
    return pdi


def negative_directional_index(close_data, high_data, low_data, period):
    """
    Negative Directional Index (-DI)
    -DI = 100 * SMMA(-DM) / ATR
    """
    catch_errors.check_for_input_len_diff(close_data, high_data, low_data)
    ndi = (100 *
           smma(negative_directional_movement(high_data, low_data), period) /
           atr(close_data, period)
           )
    return ndi


def average_directional_index(close_data, high_data, low_data, period):
    """
    Average Directional Index.
    ADX = 100 * SMMA(abs((+DI - -DI) / (+DI + -DI)))
    """
    avg_di = (abs(
              positive_directional_index(
                close_data, high_data, low_data, period) -
              negative_directional_index(
                close_data, high_data, low_data, period) /
              (positive_directional_index(
                close_data, high_data, low_data, period) +
               negative_directional_index(
                close_data, high_data, low_data, period)))
              )
    adx = 100 * smma(avg_di, period)
    return adx



import numpy as np
from py_ti import catch_errors
from py_ti.simple_moving_average import (
    simple_moving_average as sma
    )
from py_ti.typical_price import typical_price


def band_width(high_data, low_data, period):
    """
    Bandwidth
    BW = SMA(H - L)
    """
    catch_errors.check_for_input_len_diff(high_data, low_data)
    diff = np.array(high_data) - np.array(low_data)
    bw = sma(diff, period)
    return bw


def center_band(close_data, high_data, low_data, period):
    """
    Center Band
    CB = SMA(TP)
    """
    tp = typical_price(close_data, high_data, low_data)
    cb = sma(tp, period)
    return cb


def upper_band(close_data, high_data, low_data, period):
    """
    Upper Band
    UB = CB + BW
    """
    cb = center_band(close_data, high_data, low_data, period)
    bw = band_width(high_data, low_data, period)
    ub = cb + bw
    return ub


def lower_band(close_data, high_data, low_data, period):
    """
    Lower Band
    LB = CB - BW
    """
    cb = center_band(close_data, high_data, low_data, period)
    bw = band_width(high_data, low_data, period)
    lb = cb - bw
    return lb



from py_ti import catch_errors
from py_ti.simple_moving_average import (
    simple_moving_average as sma
    )


def triangular_moving_average(data, period):
    """
    Triangular Moving Average.
    TMA = SMA(SMA())
    """
    catch_errors.check_for_period_error(data, period)

    tma = sma(sma(data, period), period)
    return tma



import numpy as np
from py_ti import catch_errors


def accumulation_distribution(close_data, high_data, low_data, volume):
    """
    Accumulation/Distribution
    A/D = (Ct - Lt) - (Ht - Ct) / (Ht - Lt) * Vt + A/Dt-1
    """
    catch_errors.check_for_input_len_diff(
        close_data, high_data, low_data, volume
        )

    ad = np.zeros(len(close_data))
    for idx in range(1, len(close_data)):
        ad[idx] = (
            (((close_data[idx] - low_data[idx]) -
                (high_data[idx] - close_data[idx])) /
                (high_data[idx] - low_data[idx]) *
                volume[idx]) +
            ad[idx-1]
            )
    return ad



import numpy as np
import warnings
from py_ti import catch_errors
from py_ti.function_helper import fill_for_noncomputable_vals
from py_ti.typical_price import typical_price
from py_ti.money_flow import money_flow


def money_flow_index(close_data, high_data, low_data, volume, period):
    """
    Money Flow Index
    MFI = 100 - (100 / (1 + PMF / NMF))
    """
    catch_errors.check_for_input_len_diff(
        close_data, high_data, low_data, volume
        )
    catch_errors.check_for_period_error(close_data, period)

    mf = money_flow(close_data, high_data, low_data, volume)
    tp = typical_price(close_data, high_data, low_data)

    flow = map(lambda idx: tp[idx] > tp[idx-1], range(1, len(tp)))
    pf = map(lambda idx: mf[idx] if flow[idx] else 0, range(0, len(flow)))
    nf = map(lambda idx: mf[idx] if not flow[idx] else 0, range(0, len(flow)))

    pmf = map(
        lambda idx:
        sum(pf[idx+1-period:idx+1]),
        range(period-1, len(pf))
        )
    nmf = map(
        lambda idx:
        sum(nf[idx+1-period:idx+1]),
        range(period-1, len(nf))
        )

    # Dividing by 0 is not an issue, it turns the value into NaN which we would
    # want in that case
    with warnings.catch_warnings():
        warnings.simplefilter("ignore", category=RuntimeWarning)
        money_ratio = np.array(pmf) / np.array(nmf)

    mfi = 100 - (100 / (1 + money_ratio))

    mfi = fill_for_noncomputable_vals(close_data, mfi)

    return mfi



from py_ti import catch_errors
from py_ti.function_helper import fill_for_noncomputable_vals


def exponential_moving_average(data, period):
    """
    Exponential Moving Average.

    Formula: p0 + (1 - w) * p1 + (1 - w)^2 * p2 + (1 + w)^3 * p3 +...
                /   1 + (1 - w) + (1 - w)^2 + (1 - w)^3 +...

            where: w = 2 / (N + 1)
    """
    catch_errors.check_for_period_error(data, period)
    emas = map(
        lambda idx:
        exponential_moving_average_helper(
            data[idx - period + 1:idx + 1], period),
        range(period - 1, len(data))
        )
    emas = fill_for_noncomputable_vals(data, emas)
    return emas


def exponential_moving_average_helper(data, period):
    w = 2 / float(period + 1)
    ema_top = data[period - 1]
    ema_bottom = 1
    for idx in range(1, period):  # idx 1 to n
        ema_top += ((1 - w)**idx) * data[period - 1 - idx]
        ema_bottom += (1 - w)**idx
    ema = ema_top / ema_bottom
    return ema



import numpy as np
from py_ti import catch_errors
from py_ti.function_helper import fill_for_noncomputable_vals


def chaikin_money_flow(close_data, high_data, low_data, volume, period):
    """
    Chaikin Money Flow.
    CMF = SUM[(((Cn - Ln) - (Hn - Cn)) / (Hn - Ln)) * V] / SUM(Vn)
    """
    catch_errors.check_for_input_len_diff(
        close_data, high_data, low_data, volume)
    catch_errors.check_for_period_error(close_data, period)

    close_data = np.array(close_data)
    high_data = np.array(high_data)
    low_data = np.array(low_data)
    volume = np.array(volume)
    cmf = map(
        lambda idx:
        sum((((close_data[idx+1-period:idx+1] - low_data[idx+1-period:idx+1]) -
            (high_data[idx+1-period:idx+1] - close_data[idx+1-period:idx+1])) /
            (high_data[idx+1-period:idx+1] - low_data[idx+1-period:idx+1])) *
            volume[idx+1-period:idx+1]) / sum(volume[idx+1-period:idx+1]),
        range(period-1, len(close_data))
        )
    cmf = fill_for_noncomputable_vals(close_data, cmf)
    return cmf



import numpy as np
from py_ti import catch_errors
from py_ti.function_helper import fill_for_noncomputable_vals
from py_ti.simple_moving_average import (
    simple_moving_average as sma
    )


def upper_bollinger_band(data, period, std=2.0):
    catch_errors.check_for_period_error(data, period)

    period = int(period)
    simple_ma = sma(data, period)[period-1:]

    upper_bb = []
    for idx in range(len(data) - period + 1):
        std_dev = np.std(data[idx:idx + period])
        upper_bb.append(simple_ma[idx] + std_dev * std)
    upper_bb = fill_for_noncomputable_vals(data, upper_bb)

    return np.array(upper_bb)


def middle_bollinger_band(data, period, std=2.0):
    catch_errors.check_for_period_error(data, period)

    period = int(period)
    simple_ma = sma(data, period)[period-1:]

    middle_bb = []
    for idx in range(len(data) - period + 1):
        middle_bb.append(simple_ma[idx])
    middle_bb = fill_for_noncomputable_vals(data, middle_bb)

    return np.array(middle_bb)


def lower_bollinger_band(data, period, std=2.0):
    catch_errors.check_for_period_error(data, period)

    period = int(period)
    simple_ma = sma(data, period)[period-1:]

    lower_bb = []
    for idx in range(len(data) - period + 1):
        std_dev = np.std(data[idx:idx + period])
        lower_bb.append(simple_ma[idx] - std_dev * std)
    lower_bb = fill_for_noncomputable_vals(data, lower_bb)

    return np.array(lower_bb)


def bandwidth(data, period, std=2.0):
    catch_errors.check_for_period_error(data, period)

    period = int(period)
    bandwidth = ((upper_bollinger_band(data, period, std) -
                 lower_bollinger_band(data, period, std)) /
                 middle_bollinger_band(data, period, std)
                 )

    return bandwidth


def bb_range(data, period, std=2.0):
    catch_errors.check_for_period_error(data, period)

    period = int(period)
    bb_range = (upper_bollinger_band(data, period, std) -
                lower_bollinger_band(data, period, std)
                )
    return bb_range


def percent_bandwidth(data, period, std=2.0):
    catch_errors.check_for_period_error(data, period)

    period = int(period)
    percent_bandwidth = ((np.array(data) -
                         lower_bollinger_band(data, period, std)) /
                         bb_range(data, period, std)
                         )

    return percent_bandwidth



from py_ti import catch_errors
from py_ti.function_helper import fill_for_noncomputable_vals


def rate_of_change(data, period):
    """
    Rate of Change

    Formula:
    (Close - Close n periods ago) / (Close n periods ago) * 100
    """
    catch_errors.check_for_period_error(data, period)

    rocs = map(
        lambda idx:
        ((data[idx] - data[idx - (period - 1)]) /
         data[idx - (period - 1)]) * 100,
        range(period - 1, len(data))
        )
    rocs = fill_for_noncomputable_vals(data, rocs)
    return rocs



import numpy as np
from py_ti import catch_errors
from py_ti.typical_price import typical_price
from py_ti.simple_moving_average import (
    simple_moving_average as sma
    )


def commodity_channel_index(close_data, high_data, low_data, period):
    """
    Commodity Channel Index.
    CCI = (TP - SMA(TP)) / (0.015 * Mean Deviation)
    """
    catch_errors.check_for_input_len_diff(close_data, high_data, low_data)
    catch_errors.check_for_period_error(close_data, period)
    tp = typical_price(close_data, high_data, low_data)
    cci = ((tp - sma(tp, period)) /
           (0.015 * np.mean(np.absolute(tp - np.mean(tp)))))
    return cci



import numpy as np
from py_ti import catch_errors
from py_ti.function_helper import fill_for_noncomputable_vals


def conversion_base_line_helper(data, period):
    """
    The only real difference between TenkanSen and KijunSen is the period value
    """
    catch_errors.check_for_period_error(data, period)
    cblh = map(
        lambda idx:
        (np.max(data[idx+1-period:idx+1]) -
            np.min(data[idx+1-period:idx+1])) / 2,
        range(period-1, len(data))
        )

    cblh = fill_for_noncomputable_vals(data, cblh)
    return cblh


def tenkansen(data, period=9):
    """
    TenkanSen (Conversion Line)
    (H + L) / 2  :: default period=9
    """
    ts = conversion_base_line_helper(data, period)
    return ts


def kijunsen(data, period=26):
    """
    KijunSen (Base Line)
    (H + L) / 2  :: default period=26
    """
    ks = conversion_base_line_helper(data, period)
    return ks


def chiku_span(data):
    """
    Chiku Span (Lagging Span)
    Close shifted back 26 bars
    """
    cs = data[25::]
    return cs


def senkou_a(data):
    """
    Senkou A (Leading Span A)
    (TenkanSen + KijunSen) / 2 :: Shift Forward 26 bars
    """
    sa = (tenkansen(data) + kijunsen(data)) / 2
    # shift forward
    shift_by = np.repeat(np.nan, 26)
    sa = np.append(shift_by, sa)
    return sa


def senkou_b(data, period=52):
    """
    Senkou B (Leading Span B)
    (H + L) / 2  :: default period=52 :: shifted forward 26 bars
    """
    sb = conversion_base_line_helper(data, period)
    shift_by = np.repeat(np.nan, 26)
    sb = np.append(shift_by, sb)
    return sb



from py_ti import catch_errors
from py_ti.exponential_moving_average import (
    exponential_moving_average as ema
    )


def price_oscillator(data, short_period, long_period):
    """
    Price Oscillator
    (short EMA - long EMA / long EMA) * 100
    """
    catch_errors.check_for_period_error(data, short_period)
    catch_errors.check_for_period_error(data, long_period)

    ema_short = ema(data, short_period)
    ema_long = ema(data, long_period)

    po = ((ema_short - ema_long) / ema_long) * 100
    return po



import numpy as np
import warnings
from py_ti import catch_errors


def chande_momentum_oscillator(close_data, period):
    """
    Chande Momentum Oscillator.
    """
    catch_errors.check_for_period_error(close_data, period)

    close_data = np.array(close_data)

    moving_period_diffs = map(
        lambda idx:
            map(lambda i:
                (close_data[idx+1-period:idx+1][i] -
                 close_data[idx+1-period:idx+1][i-1]),
                range(1, len(close_data[idx+1-period:idx+1]))),
            range(0, len(close_data))
        )

    sum_up = []
    sum_down = []
    for period_diffs in moving_period_diffs:
        ups = map(lambda val: val if val > 0 else 0, period_diffs)
        sum_up.append(sum(ups))
        downs = map(lambda val: abs(val) if val < 0 else 0, period_diffs)
        sum_down.append(sum(downs))

    sum_up = np.array(sum_up)
    sum_down = np.array(sum_down)
    # numpy is able to handle dividing by zero and makes those calculations
    # nans which is what we want, so we safely suppress the RuntimeWarning
    with warnings.catch_warnings():
        warnings.simplefilter("ignore", category=RuntimeWarning)
        cmo = 100 * ((sum_up - sum_down) / (sum_up + sum_down))
    return cmo



import numpy as np
from py_ti import catch_errors
from py_ti.function_helper import fill_for_noncomputable_vals


def relative_strength_index(data, period):
    """
    Relative Strength Index.

    RSI = 100 - (100 / 1 + (prevGain/prevLoss))
    """
    catch_errors.check_for_period_error(data, period)

    period = int(period)
    changes = map(
        lambda data_tup:
        data_tup[1] - data_tup[0],
        zip(data[::1], data[1::1])
        )

    filtered_gain = map(lambda val: val < 0, changes)
    gains = map(
        lambda idx:
        0 if filtered_gain[idx] is True else changes[idx],
        range(0, len(filtered_gain))
        )

    filtered_loss = map(lambda val: val > 0, changes)
    losses = map(
        lambda idx:
        0 if filtered_loss[idx] is True else abs(changes[idx]),
        range(0, len(filtered_loss))
        )

    avg_gain = np.mean(gains[:period])
    avg_loss = np.mean(losses[:period])

    rsi = []
    if avg_loss == 0:
        rsi.append(100)
    else:
        rs = avg_gain / avg_loss
        rsi.append(100 - (100 / (1 + rs)))

    for idx in range(1, len(data) - period):
        avg_gain = ((avg_gain * (period - 1) +
                    gains[idx + (period - 1)]) / period)
        avg_loss = ((avg_loss * (period - 1) +
                    losses[idx + (period - 1)]) / period)

        if avg_loss == 0:
            rsi.append(100)
        else:
            rs = avg_gain / avg_loss
            rsi.append(100 - (100 / (1 + rs)))

    rsi = fill_for_noncomputable_vals(data, rsi)

    return rsi



import numpy as np
from py_ti import catch_errors


def volume_index_helper(vi, idx, close_data, volume):
    return (vi[idx-1] + (((close_data[idx] - close_data[idx-1]) /
            close_data[idx-1]) * vi[idx-1])
            )


def positive_volume_index(close_data, volume):
    """
    Positive Volume Index (PVI)
    PVI0 = 1
    IF Vt > Vt-1
        PVIt = PVIt-1 + (CLOSEt - CLOSEt-1 / CLOSEt-1 * PVIt-1)
    ELSE:
        PVIt = PVIt-1
    """
    catch_errors.check_for_input_len_diff(close_data, volume)
    pvi = np.zeros(len(volume))
    pvi[0] = 1
    for idx in range(1, len(volume)):
        if volume[idx] > volume[idx-1]:
            pvi[idx] = volume_index_helper(pvi, idx, close_data, volume)
        else:
            pvi[idx] = pvi[idx-1]
    return pvi


def negative_volume_index(close_data, volume):
    """
    Negative Volume Index (NVI)
    NVI0 = 1
    IF Vt < Vt-1
        NVIt = NVIt-1 + (CLOSEt - CLOSEt-1 / CLOSEt-1 * NVIt-1)
    ELSE:
        NVIt = NVIt-1
    """
    catch_errors.check_for_input_len_diff(close_data, volume)
    nvi = np.zeros(len(volume))
    nvi[0] = 1
    for idx in range(1, len(volume)):
        if volume[idx] < volume[idx-1]:
            nvi[idx] = volume_index_helper(nvi, idx, close_data, volume)
        else:
            nvi[idx] = nvi[idx-1]
    return nvi



from py_ti import catch_errors
from py_ti.function_helper import fill_for_noncomputable_vals


def linear_weighted_moving_average(data, period):
    """
    Linear Weighted Moving Average.
    LWMA = SUM(DATA[i]) * i / SUM(i)
    """
    catch_errors.check_for_period_error(data, period)

    idx_period = range(1, period+1)
    lwma = map(
        lambda idx:
        (sum([i * idx_period[data[idx-(period-1):idx+1].index(i)]
              for i in data[idx-(period-1):idx+1]])) /
            sum(range(1, len(data[idx+1-period:idx+1])+1)),
        range(period-1, len(data))
        )
    lwma = fill_for_noncomputable_vals(data, lwma)
    return lwma



from py_ti import catch_errors
from py_ti.exponential_moving_average import (
    exponential_moving_average as ema
    )


def upper_price_channel(data, period, upper_percent):
    """
    Upper Price Channel
    """
    catch_errors.check_for_period_error(data, period)

    emas = ema(data, period)
    upper_channel = [val * (1+float(upper_percent)/100) for val in emas]
    return upper_channel


def lower_price_channel(data, period, lower_percent):
    """
    Lower Price Channel
    """
    catch_errors.check_for_period_error(data, period)

    emas = ema(data, period)
    lower_channel = [val * (1-float(lower_percent)/100) for val in emas]
    return lower_channel



from py_ti import catch_errors
from py_ti.typical_price import typical_price as tp


def money_flow(close_data, high_data, low_data, volume):
    """
    Money Flow.
    MF = VOLUME * TYPICAL PRICE
    """
    catch_errors.check_for_input_len_diff(
        close_data, high_data, low_data, volume
        )
    mf = volume * tp(close_data, high_data, low_data)
    return mf



from py_ti import catch_errors
from py_ti.function_helper import fill_for_noncomputable_vals


def weighted_moving_average(data, period):
    """
    Weighted Moving Average

    Formula:
    (P1 + 2 P2 + 3 P3 + ... + n Pn) / K
    where K = (1+2+...+n) = n(n+1)/2 and Pn is the most recent price
    """
    catch_errors.check_for_period_error(data, period)
    k = (period * (period + 1)) / 2.0

    wmas = []
    for idx in range(0, len(data)-period+1):
        product = map(
            lambda period_idx: data[idx + period_idx] * (period_idx + 1),
            range(0, period)
            )
        wma = sum(product) / k
        wmas.append(wma)
    wmas = fill_for_noncomputable_vals(data, wmas)

    return wmas



import numpy as np
from py_ti import catch_errors
from py_ti.function_helper import fill_for_noncomputable_vals


def smoothed_moving_average(data, period):
    """
    Smoothed Moving Average.
    """
    catch_errors.check_for_period_error(data, period)
    smma = map(
        lambda idx:
        ((np.mean(data[idx-(period-1):idx+1]) -
         (np.mean(data[idx-(period-1):idx+1])/period) +
         data[idx])/period),
        range(0, len(data))
        )
    smma = fill_for_noncomputable_vals(data, smma)
    return smma



import numpy as np
from py_ti import catch_errors
from py_ti.function_helper import fill_for_noncomputable_vals


def standard_variance(data, period):
    """
    Standard Variance.

    (Ct - AVGt)^2 / N
    """
    catch_errors.check_for_period_error(data, period)
    sv = map(
        lambda idx:
        np.var(data[idx+1-period:idx+1], ddof=1),
        range(period-1, len(data))
        )
    sv = fill_for_noncomputable_vals(data, sv)

    return sv



import numpy as np
from py_ti import catch_errors
from py_ti.function_helper import fill_for_noncomputable_vals


def detrended_price_oscillator(data, period):
    """
    Detrended Price Oscillator.
    DPO = DATA[i] - Avg(DATA[period/2 + 1])
    """
    catch_errors.check_for_period_error(data, period)
    period = int(period)
    dop = map(
        lambda idx:
        data[idx] - np.mean(data[idx+1-((period/2)+1):idx+1]),
        range(period-1, len(data))
        )
    dop = fill_for_noncomputable_vals(data, dop)
    return dop



import numpy as np
from py_ti import catch_errors
from py_ti.weighted_moving_average import (
    weighted_moving_average as wma
    )


def hull_moving_average(data, period):
    """
    Hull Moving Average.
    HMA = WMA(2*WMA(n/2) - WMA(n)), sqrt(n)
    """
    catch_errors.check_for_period_error(data, period)
    hma = wma(
        2 * wma(data, int(period/2)) - wma(data, period), int(np.sqrt(period))
        )
    return hma



from py_ti.standard_deviation import standard_deviation as sd
from py_ti.standard_variance import standard_variance as sv


def volatility(data, period):
    """
    Volatility.
    SDt / SVt
    """
    volatility = sd(data, period) / sv(data, period)
    return volatility



import numpy as np
from py_ti import catch_errors
from py_ti.average_true_range import (
    average_true_range as atr
    )


def average_true_range_percent(close_data, period):
    """
    Average True Range Percent.
    ATRP = (ATR / CLOSE) * 100
    """
    catch_errors.check_for_period_error(close_data, period)
    atrp = (atr(close_data, period) / np.array(close_data)) * 100
    return atrp



import numpy as np
from py_ti.true_range import true_range


def average_true_range(close_data, period):
    """
    Average True Range.
    ATRt = ATRt-1 * (n - 1) + TRt / n
    """
    tr = true_range(close_data, period)
    atr = np.zeros(len(tr))
    atr[0:period-1] = tr[0:period-1]
    atr[period-1:period-1+period-1] = tr[0:period-1]
    atr[period-1+period-1] = ((1 / float(period)) *
                              sum(tr[period-1:period-1+period])
                              )
    for idx in range(period-1+period, len(tr)):
        atr[idx] = (atr[idx-1] * (period - 1) + tr[idx]) / period
    return atr



import numpy as np
from py_ti import catch_errors
from py_ti.function_helper import fill_for_noncomputable_vals


def aroon_up(data, period):
    """
    Aroon Up.
    AROONUP = (((PERIOD) - (PERIODS since PERIOD high)) / (PERIOD)) * 100
    """
    catch_errors.check_for_period_error(data, period)
    period = int(period)

    aroon_up = map(
        lambda idx:
        ((period -
            data[idx+1-period:idx+1].index(np.max(data[idx+1-period:idx+1]))) /
            period) * 100,
        range(period-1, len(data))
        )
    aroon_up = fill_for_noncomputable_vals(data, aroon_up)
    return aroon_up


def aroon_down(data, period):
    """
    Aroon Down.
    AROONDWN = (((PERIOD) - (PERIODS SINCE PERIOD LOW)) / (PERIOD)) * 100
    """
    catch_errors.check_for_period_error(data, period)
    period = int(period)

    aroon_down = map(
        lambda idx:
        ((period -
            data[idx+1-period:idx+1].index(np.min(data[idx+1-period:idx+1]))) /
            period) * 100,
        range(period-1, len(data)))
    aroon_down = fill_for_noncomputable_vals(data, aroon_down)
    return aroon_down



from py_ti import catch_errors
from py_ti.function_helper import fill_for_noncomputable_vals


def momentum(data, period):
    """
    Momentum.
    DATA[i] - DATA[i - period]
    """
    catch_errors.check_for_period_error(data, period)

    momentum = map(
        lambda idx:
        data[idx] - data[idx+1-period],
        range(period-1, len(data))
        )
    momentum = fill_for_noncomputable_vals(data, momentum)
    return momentum



from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))


setup(
    name="pyti",

    version='0.1.1',

    description='Technical Indicator Library',
    long_description="This library contains various financial technical "
                     "indicators that can be used to analyze financial data.",

    url='https://github.com/kylejusticemagnuson/pyti',

    author='Kyle Justice Magnuson',
    author_email='kyle@collectiveidea.com',

    license='MIT',

    classifiers=[
        'Development Status :: 4 - Beta',

        'Intended Audience :: Developers',
        'Topic :: Software Development',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 2.7',
    ],

    keywords='financial technical indicator functions',

    packages=find_packages(exclude=['tests']),

    install_requires=['numpy'],

)



import numpy as np


def fill_for_noncomputable_vals(input_data, result_data):
    non_computable_values = np.repeat(
        np.nan, len(input_data) - len(result_data)
        )
    filled_result_data = np.append(non_computable_values, result_data)
    return filled_result_data



import numpy as np
from py_ti import catch_errors
from py_ti.function_helper import fill_for_noncomputable_vals
from py_ti.exponential_moving_average import (
    exponential_moving_average as ema
    )


def double_smoothed_stochastic(data, period):
    """
    Double Smoothed Stochastic
    """
    catch_errors.check_for_period_error(data, period)
    lows = map(
        lambda idx:
        data[idx] - np.min(data[idx+1-period:idx+1]),
        range(period-1, len(data))
        )
    sm_lows = ema(ema(lows, period), period)
    highs = map(
        lambda idx:
        np.max(data[idx+1-period:idx+1]) - np.min(data[idx+1-period:idx+1]),
        range(period-1, len(data))
        )
    sm_highs = ema(ema(highs, period), period)
    dss = (sm_lows / sm_highs) * 100

    dss = fill_for_noncomputable_vals(data, dss)
    return dss



import numpy as np
import warnings
from py_ti.function_helper import fill_for_noncomputable_vals
from py_ti import catch_errors


def simple_moving_average(data, period):
    """
    Simple Moving Average.

    Formula: SUM(data / N)
    """
    catch_errors.check_for_period_error(data, period)
    # Mean of Empty Slice RuntimeWarning doesn't affect output so it is
    # supressed
    with warnings.catch_warnings():
        warnings.simplefilter("ignore", category=RuntimeWarning)
        sma = map(
            lambda idx:
            np.mean(data[idx-(period-1):idx+1]),
            range(0, len(data))
            )
    sma = fill_for_noncomputable_vals(data, sma)
    return sma



import numpy as np
from py_ti import catch_errors
from py_ti.function_helper import fill_for_noncomputable_vals


def true_range(close_data, period):
    """
    True Range.
    TRt = MAX(abs(Ht - Lt), abs(Ht - Ct-1), abs(Lt - Ct-1))
    """
    catch_errors.check_for_period_error(close_data, period)

    tr = map(
        lambda idx:
        np.max([np.max(close_data[idx+1-period:idx+1]) -
                np.min(close_data[idx+1-period:idx+1]),
                abs(np.max(close_data[idx+1-period:idx+1]) -
                close_data[idx-1]),
                abs(np.min(close_data[idx+1-period:idx+1]) -
                close_data[idx-1])]),
        range(period-1, len(close_data))
        )
    tr = fill_for_noncomputable_vals(close_data, tr)
    return tr



import numpy as np
from py_ti import catch_errors
from py_ti.function_helper import fill_for_noncomputable_vals


def vertical_horizontal_filter(data, period):
    """
    Vertical Horizontal Filter
    ABS(pHIGH - pLOW) / SUM(ABS(Pi - Pi-1))
    """
    catch_errors.check_for_period_error(data, period)

    vhf = map(
        lambda idx:
        abs(np.max(data[idx+1-period:idx+1]) -
            np.min(data[idx+1-period:idx+1])) /
        sum(map(
            lambda i:
            abs(data[idx+1-period:idx+1][i] - data[idx+1-period:idx+1][i-1]),
            range(0, len(data[idx+1-period:idx+1])))),
        range(period - 1, len(data))
        )

    vhf = fill_for_noncomputable_vals(data, vhf)
    return vhf




def check_for_period_error(data, period):
    period = int(period)
    data_len = len(data)
    if data_len < period:
        raise Exception("Error: data_len < period")


def check_for_input_len_diff(*args):
    arrays_len = map(lambda arr: len(arr), args)
    if not all(a == arrays_len[0] for a in arrays_len):
        err_msg = ("Error: mismatched data lengths, check to ensure that all "
                   "input data is the same length and valid")
        raise Exception(err_msg)



from py_ti import catch_errors
from py_ti.exponential_moving_average import (
    exponential_moving_average as ema
    )


def moving_average_convergence_divergence(data, short_period, long_period):
    """
    Moving Average Convergence Divergence.
    EMA(DATA, P1) - EMA(DATA, P2)
    """
    catch_errors.check_for_period_error(data, short_period)
    catch_errors.check_for_period_error(data, long_period)

    macd = ema(data, short_period) - ema(data, long_period)
    return macd
